import { Directive, ElementRef, Renderer2, Input, NgModule } from '@angular/core';
import { isAndroid, isIOS, addWeakEventListener, View, removeWeakEventListener } from '@nativescript/core';
import { Shadow } from '@imagene.me/nativescript-shadow';

class NativeShadowDirective {
    constructor(el, render) {
        this.el = el;
        this.render = render;
        this.loaded = false;
        this.initialized = false;
        this.eventsBound = false;
        this.monkeyPatch = val => {
            this.previousNSFn.call(this.el.nativeElement, val);
            this.applyShadow();
        };
        if (isAndroid) {
            this.originalNSFn = this.el.nativeElement._redrawNativeBackground; //always store the original method
        }
    }
    ngOnInit() {
        if (!this.initialized) {
            this.initialized = true;
            this.initializeCommonData();
            if (isAndroid) {
                this.initializeAndroidData();
            }
            else if (isIOS) {
                this.initializeIOSData();
            }
            if (this.shadow && this.shadow.elevation) {
                if (isAndroid) {
                    this.loadFromAndroidData(this.shadow);
                }
                else if (isIOS) {
                    this.loadFromIOSData(this.shadow);
                }
            }
            this.bindEvents();
        }
    }
    ngOnDestroy() {
        if (this.initialized) {
            this.unbindEvents();
            this.initialized = false;
        }
    }
    // NS ListViews create elements dynamically
    // loaded and unloaded are called before angular is "ready"
    // https://github.com/NativeScript/nativescript-angular/issues/1221#issuecomment-422813111
    // So we ensure we're running loaded/unloaded events outside of angular
    bindEvents() {
        if (!this.eventsBound) {
            addWeakEventListener(this.el.nativeElement, View.loadedEvent, this.onLoaded, this);
            addWeakEventListener(this.el.nativeElement, View.unloadedEvent, this.onUnloaded, this);
            this.eventsBound = true;
            // in some cases, the element is already loaded by time of binding
            if (this.el.nativeElement.isLoaded) {
                this.onLoaded();
            }
        }
    }
    unbindEvents() {
        if (this.eventsBound) {
            removeWeakEventListener(this.el.nativeElement, View.loadedEvent, this.onLoaded, this);
            removeWeakEventListener(this.el.nativeElement, View.unloadedEvent, this.onUnloaded, this);
            this.eventsBound = false;
        }
    }
    onLoaded() {
        this.loaded = true;
        // Weirdly ngOnInit isn't called on iOS on demo app
        // Managed to get it working on iOS when applying to
        // FlexboxLayout, but on the demo app, we apply to a
        // Label, and, for that case, ngOnInit isn't called
        // This is just enforcing the Directive is initialized
        // before calling this.applyShadow()
        if (!this.initialized) {
            this.ngOnInit();
        }
        this.applyShadow();
        if (isAndroid) {
            this.previousNSFn = this.el.nativeElement._redrawNativeBackground; // just to maintain compatibility with other patches
            this.el.nativeElement._redrawNativeBackground = this.monkeyPatch;
        }
    }
    addIosWrapper() {
        var _a;
        if (isIOS) {
            const originalElement = this.el.nativeElement;
            this.iosShadowRapper = this.render.createElement('GridLayout');
            this.cloneViewPositionProperties(originalElement, this.iosShadowRapper);
            // wrappingElement.cssClasses = mainElement.cssClasses;
            const parent = originalElement.parentNode;
            this.render.insertBefore(parent, this.iosShadowRapper, originalElement);
            this.render.removeChild(parent, originalElement);
            if (originalElement.parentNode) {
                (_a = originalElement.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(originalElement);
            }
            this.render.appendChild(this.iosShadowRapper, originalElement);
        }
    }
    onUnloaded() {
        this.loaded = false;
        if (isAndroid) {
            this.el.nativeElement._redrawNativeBackground = this.originalNSFn; // always revert to the original method
        }
    }
    ngAfterViewInit() {
        this.addIosWrapper();
    }
    ngOnChanges(changes) {
        if (this.loaded &&
            !!changes &&
            (changes.hasOwnProperty('shadow') ||
                changes.hasOwnProperty('elevation') ||
                changes.hasOwnProperty('pressedElevation') ||
                changes.hasOwnProperty('shape') ||
                changes.hasOwnProperty('bgcolor') ||
                changes.hasOwnProperty('cornerRadius') ||
                changes.hasOwnProperty('pressedTranslationZ') ||
                changes.hasOwnProperty('forcePressAnimation') ||
                changes.hasOwnProperty('translationZ') ||
                changes.hasOwnProperty('maskToBounds') ||
                changes.hasOwnProperty('shadowColor') ||
                changes.hasOwnProperty('shadowOffset') ||
                changes.hasOwnProperty('shadowOpacity') ||
                changes.hasOwnProperty('shadowRadius') ||
                changes.hasOwnProperty('rasterize') ||
                changes.hasOwnProperty('useShadowMap'))) {
            if (changes.hasOwnProperty('shadow') &&
                !changes.hasOwnProperty('elevation') &&
                typeof changes.shadow.currentValue === 'number') {
                this.elevation = changes.shadow.currentValue;
            }
            if (changes.shadow && changes.shadow.currentValue.elevation) {
                if (isAndroid) {
                    this.loadFromAndroidData(this.shadow);
                }
                else if (isIOS) {
                    this.loadFromIOSData(this.shadow);
                }
            }
            this.applyShadow();
        }
    }
    applyShadow() {
        if (this.shadow === null ||
            this.shadow === undefined ||
            (this.shadow === '' && !this.elevation)) {
            return;
        }
        // For shadows to be shown on Android the SDK has to be greater
        // or equal than 21, lower SDK means no setElevation method is available
        if (isAndroid) {
            if (android.os.Build.VERSION.SDK_INT < 21) {
                return;
            }
        }
        const viewToApplyShadowTo = isIOS
            ? this.iosShadowRapper
            : this.el.nativeElement;
        if (viewToApplyShadowTo) {
            Shadow.apply(viewToApplyShadowTo, {
                elevation: this.elevation,
                pressedElevation: this.pressedElevation,
                shape: this.shape,
                bgcolor: this.bgcolor,
                cornerRadius: this.cornerRadius,
                translationZ: this.translationZ,
                pressedTranslationZ: this.pressedTranslationZ,
                forcePressAnimation: this.forcePressAnimation,
                maskToBounds: this.maskToBounds,
                shadowColor: this.shadowColor,
                shadowOffset: this.shadowOffset,
                shadowOpacity: this.shadowOpacity,
                shadowRadius: this.shadowRadius,
                rasterize: this.rasterize,
                useShadowPath: this.useShadowPath
            });
        }
    }
    initializeCommonData() {
        const tShadow = typeof this.shadow;
        if ((tShadow === 'string' || tShadow === 'number') && !this.elevation) {
            this.elevation = this.shadow ? parseInt(this.shadow, 10) : 2;
        }
        const tElevation = typeof this.elevation;
        if (tElevation === 'string' || tElevation === 'number') {
            this.elevation = this.elevation
                ? parseInt(this.elevation, 10)
                : 2;
        }
    }
    initializeAndroidData() {
        if (typeof this.cornerRadius === 'string') {
            this.cornerRadius = parseInt(this.cornerRadius, 10);
        }
        if (typeof this.translationZ === 'string') {
            this.translationZ = parseInt(this.translationZ, 10);
        }
    }
    initializeIOSData() {
        if (typeof this.shadowOffset === 'string') {
            this.shadowOffset = parseFloat(this.shadowOffset);
        }
        if (typeof this.shadowOpacity === 'string') {
            this.shadowOpacity = parseFloat(this.shadowOpacity);
        }
        if (typeof this.shadowRadius === 'string') {
            this.shadowRadius = parseFloat(this.shadowRadius);
        }
    }
    loadFromAndroidData(data) {
        this.elevation = data.elevation || this.elevation;
        this.shape = data.shape || this.shape;
        this.bgcolor = data.bgcolor || this.bgcolor;
        this.cornerRadius = data.cornerRadius || this.cornerRadius;
        this.translationZ = data.translationZ || this.translationZ;
    }
    loadFromIOSData(data) {
        this.maskToBounds = data.maskToBounds || this.maskToBounds;
        this.shadowColor = data.shadowColor || this.shadowColor;
        this.shadowOffset = data.shadowOffset || this.shadowOffset;
        this.shadowOpacity = data.shadowOpacity || this.shadowOpacity;
        this.shadowRadius = data.shadowRadius || this.shadowRadius;
        this.rasterize = data.rasterize || this.rasterize;
        this.useShadowPath = data.useShadowPath || this.useShadowPath;
    }
    cloneViewPositionProperties(sourceView, targetView) {
        if (sourceView.row > 0) {
            targetView.row = sourceView.row;
        }
        if (sourceView.rowSpan > 0) {
            targetView.rowSpan = sourceView.rowSpan;
        }
        if (sourceView.col > 0) {
            targetView.col = sourceView.col;
        }
        if (sourceView.colSpan > 0) {
            targetView.colSpan = sourceView.colSpan;
        }
        if (sourceView.verticalAlignment) {
            targetView.verticalAlignment = sourceView.verticalAlignment;
        }
        if (sourceView.horizontalAlignment) {
            targetView.horizontalAlignment = sourceView.horizontalAlignment;
        }
    }
}
NativeShadowDirective.decorators = [
    { type: Directive, args: [{ selector: '[shadow]' },] }
];
NativeShadowDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
NativeShadowDirective.propDecorators = {
    shadow: [{ type: Input }],
    elevation: [{ type: Input }],
    pressedElevation: [{ type: Input }],
    shape: [{ type: Input }],
    bgcolor: [{ type: Input }],
    cornerRadius: [{ type: Input }],
    translationZ: [{ type: Input }],
    pressedTranslationZ: [{ type: Input }],
    forcePressAnimation: [{ type: Input }],
    maskToBounds: [{ type: Input }],
    shadowColor: [{ type: Input }],
    shadowOffset: [{ type: Input }],
    shadowOpacity: [{ type: Input }],
    shadowRadius: [{ type: Input }],
    useShadowPath: [{ type: Input }],
    rasterize: [{ type: Input }]
};

class NgShadowModule {
}
NgShadowModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                declarations: [NativeShadowDirective],
                exports: [NativeShadowDirective],
                providers: [],
            },] }
];
// Uncomment this line if the package provides a custom view component
// registerElement('NativescriptShadow', () => NativescriptShadow);

/**
 * Generated bundle index. Do not edit.
 */

export { NativeShadowDirective, NgShadowModule };
//# sourceMappingURL=nativescript-shadow-angular.js.map
